@@ -1,8 +1,6 @@
 import functools
-import string
 from collections import defaultdict
 from pathlib import Path
-from random import Random
 from typing import *
 
 from orkestra.interfaces import (
@@ -10,20 +8,17 @@ from orkestra.interfaces import (
     Runtime,
     LambdaInvocationType,
     IntegrationPattern,
+    Tracing,
 )
-from orkestra.utils import coerce
+from orkestra.utils import coerce, cdk_patch
+from orkestra.exceptions import CompositionError
 
 OptionalFn = Optional[Union[Callable, Iterable[Callable]]]
 
-random = Random(0)
 
 _id_map = defaultdict(lambda: 1)
 
 
-def _sample(k=4):
-    return "".join(random.sample(string.hexdigits, k))
-
-
 def _incremental_id(id):
 
     mapped = _id_map[id]
@@ -45,7 +40,7 @@ class Compose:
         capture_error=True,
         raise_on_empty_metrics=False,
         capture_cold_start_metric=True,
-        default_dimentions=None,
+        default_dimensions=None,
         model=None,
         envelope=None,
         timeout: Optional[Duration] = None,
@@ -67,6 +62,7 @@ class Compose:
         heartbeat: Optional[Duration] = None,
         integration_pattern: Optional[IntegrationPattern] = None,
         sfn_timeout: Optional[Duration] = None,
+        tracing: Optional[Tracing] = None,
         **aws_lambda_constructor_kwargs,
     ):
         """
@@ -81,7 +77,7 @@ class Compose:
             capture_error: passed to aws_lambda_powertools.Tracer
             raise_on_empty_metrics: passed to aws_lambda_powertools.Metrics
             capture_cold_start_metric: passed to aws_lambda_powertools.Metrics
-            default_dimentions: passed to aws_lambda_powertools.Metrics
+            default_dimensions: passed to aws_lambda_powertools.Metrics
             model: passed to aws_lambda_powertools.utilities.parser.event_parser
             envelope: passed to aws_lambda_powertools.utilities.parser.event_parser
             runtime: the python runtime to use for the lambda
@@ -102,6 +98,7 @@ class Compose:
             heartbeat: Timeout for the heartbeat. Default: - None
             integration_pattern: AWS Step Functions integrates with services directly in the Amazon States Language. You can control these AWS services using service integration patterns Default: IntegrationPattern.REQUEST_RESPONSE
             sfn_timeout: Timeout for the state machine. Default: - None
+            tracing: Enable AWS X-Ray Tracing for Lambda Function. Default: Tracing.Enabled
             **aws_lambda_constructor_kwargs: pass directly to sfn.PythonFunction
 
         For cdk params see https://docs.aws.amazon.com/cdk/api/latest/python/modules.html
@@ -149,9 +146,14 @@ class Compose:
 
             module = func.__module__.split(".")
 
-            self.entry = str(Path(*module).parent)
-            self.index = Path(*module).name + ".py"
-            self.handler = func.__name__
+            self.aws_lambda_constructor_kwargs.update(
+                entry=str(Path(*module).parent),
+                index=Path(*module).name + ".py",
+                handler=func.__name__,
+                timeout=timeout,
+                runtime=runtime,
+                tracing=tracing,
+            )
 
         self.enable_powertools = enable_powertools
 
@@ -161,7 +163,7 @@ class Compose:
             capture_response=capture_response,
             capture_cold_start_metric=capture_cold_start_metric,
             raise_on_empty_metrics=raise_on_empty_metrics,
-            default_dimentions=default_dimentions,
+            default_dimensions=default_dimensions,
             model=model,
             envelope=envelope,
         )
@@ -212,13 +214,7 @@ class Compose:
         else:
             func = self.func.__name__ if self.func is not None else None
 
-        return (
-            "Task("
-            f"func={func}, "
-            f"aws_lambda_constructor_kwargs={self.aws_lambda_constructor_kwargs}, "
-            f"len_downstream={len(self.downstream)}"
-            ")"
-        )
+        return f"Compose(func={func})"
 
     def __rshift__(self, right):
         right = (
@@ -232,41 +228,36 @@ class Compose:
         composable: "Compose",
         scope,
         id=None,
-        function_name=None,
-        tracing=None,
-        runtime=None,
-        dead_letter_queue_enabled=False,
         **kwargs,
     ):
 
         from aws_cdk import aws_lambda, aws_lambda_python
 
-        tracing = tracing or aws_lambda.Tracing.ACTIVE
-        runtime = runtime or aws_lambda.Runtime.PYTHON_3_8
+        keyword_args = {
+            **composable.aws_lambda_constructor_kwargs,
+            **kwargs,
+        }
 
-        keyword_args = dict(
-            entry=composable.entry,
-            handler=composable.handler,
-            index=composable.index,
-            function_name=function_name,
-            runtime=runtime,
-            tracing=tracing,
-            dead_letter_queue_enabled=dead_letter_queue_enabled,
+        cdk_patch(
+            keyword_args,
+            "tracing",
+            "runtime",
+            "timeout",
         )
 
-        keyword_args.update(kwargs)
+        if keyword_args.get("runtime") is None:
 
-        keyword_args.update(composable.aws_lambda_constructor_kwargs)
+            keyword_args.update(
+                runtime=aws_lambda.Runtime.PYTHON_3_8,
+            )
 
-        id = id or f"{composable.func.__name__}_fn_{_sample()}"
+        if keyword_args.get("tracing") is None:
 
-        if composable.timeout is not None:
+            keyword_args.update(
+                tracing=aws_lambda.Tracing.ACTIVE,
+            )
 
-            keyword_args.update(timeout=composable.timeout.construct)
-
-        if composable.runtime is not None:
-
-            keyword_args.update(runtime=composable.runtime.construct)
+        id = id or _incremental_id(composable.func.__name__ + "_fn")
 
         return aws_lambda_python.PythonFunction(
             scope,
@@ -278,21 +269,24 @@ class Compose:
         self,
         scope,
         id=None,
-        function_name=None,
-        tracing=None,
-        runtime=None,
-        dead_letter_queue_enabled=False,
         **kwargs,
     ):
+        """
+        Return lambda cdk construct.
+
+        Args:
+            scope:
+            id:
+            **kwargs:
+
+        Returns (aws_cdk.aws_lambda_python.PythonFunction): python lambda
+
+        """
 
         return self._render_lambda(
             self,
             scope,
             id=id,
-            function_name=function_name,
-            tracing=tracing,
-            runtime=runtime,
-            dead_letter_queue_enabled=dead_letter_queue_enabled,
             **kwargs,
         )
 
@@ -304,6 +298,9 @@ class Compose:
         function_name=None,
         **kwargs,
     ):
+        """
+        Return cdk step function task construct.
+        """
 
         from aws_cdk import aws_stepfunctions as sfn
         from aws_cdk import aws_stepfunctions_tasks as sfn_tasks
@@ -338,6 +335,13 @@ class Compose:
                 }
             )
 
+            cdk_patch(
+                keyword_args,
+                "integration_pattern",
+                "timeout",
+                "invocation_type",
+            )
+
             invoke_lambda = sfn_tasks.LambdaInvoke(
                 scope,
                 f"invoke_{id}",
@@ -370,6 +374,13 @@ class Compose:
                 }
             )
 
+            cdk_patch(
+                keyword_args,
+                "integration_pattern",
+                "timeout",
+                "invocation_type",
+            )
+
             task = sfn_tasks.LambdaInvoke(
                 scope,
                 id,
@@ -378,9 +389,13 @@ class Compose:
 
         else:
 
+            id = "parallelize " + (
+                "".join([c.func.__name__ for c in self.func])
+            )
+
             task = sfn.Parallel(
                 scope,
-                "parallelize {}".format([c.func.__name__ for c in self.func]),
+                _incremental_id(id),
             )
 
             for fn in self.func:
@@ -402,6 +417,13 @@ class Compose:
                     }
                 )
 
+                cdk_patch(
+                    keyword_args,
+                    "integration_pattern",
+                    "timeout",
+                    "invocation_type",
+                )
+
                 branch = sfn_tasks.LambdaInvoke(
                     scope,
                     _incremental_id(fn.func.__name__),
@@ -422,15 +444,32 @@ class Compose:
         self,
         scope,
         definition=None,
+        previous=None,
     ):
+        """
+        Return automagically composed cdk state machine definition.
+        """
+        previous = previous or []
+
+        if self in previous:
+
+            raise CompositionError(
+                f"Failed to compose {self}. Composition using >> must be acyclic."
+            )
 
         task = self.task(scope)
 
         definition = task if definition is None else definition.next(task)
 
         if self.downstream:
+
             for c in self.downstream:
-                c.definition(scope, definition=definition)
+
+                c.definition(
+                    scope,
+                    definition=definition,
+                    previous=previous + [self],
+                )
 
         return definition
 
@@ -442,10 +481,23 @@ class Compose:
         state_machine_name=None,
         **kwargs,
     ):
+        """
+        Return step functions state machine cdk construct.
+
+        Args:
+            scope:
+            id:
+            tracing_enabled:
+            state_machine_name:
+            **kwargs:
+
+        Returns:
+
+        """
 
         from aws_cdk import aws_stepfunctions as sfn
 
-        id = id or f"{self.func.__name__}_sfn_{_sample()}"
+        id = id or _incremental_id(f"{self.func.__name__}_sfn")
 
         return sfn.StateMachine(
             scope,
@@ -472,10 +524,31 @@ class Compose:
         dead_letter_queue_enabled=False,
         **kwargs,
     ):
+        """
+        Schedule lambda or state machine to run on interval using EventBridge scheduled event rule.
+
+        Args:
+            scope:
+            id:
+            expression:
+            day:
+            hour:
+            minute:
+            month:
+            week_day:
+            year:
+            function_name:
+            state_machine_name:
+            dead_letter_queue_enabled:
+            **kwargs:
+
+        Returns:
+
+        """
         from aws_cdk import aws_events as eventbridge
         from aws_cdk import aws_events_targets as eventbridge_targets
 
-        id = id or f"{self.func.__name__}_sched_{_sample()}"
+        id = id or _incremental_id(f"{self.func.__name__}_sched")
 
         if expression is not None:
             schedule = eventbridge.Schedule.expression(expression)
@@ -522,7 +595,7 @@ def powertools(
     capture_error=True,
     raise_on_empty_metrics=False,
     capture_cold_start_metric=True,
-    default_dimentions=None,
+    default_dimensions=None,
     model=None,
     envelope=None,
 ):
@@ -536,7 +609,7 @@ def powertools(
         capture_error: passed to aws_lambda_powertools.Tracer
         raise_on_empty_metrics: passed to aws_lambda_powertools.Metrics
         capture_cold_start_metric: passed to aws_lambda_powertools.Metrics
-        default_dimentions: passed to aws_lambda_powertools.Metrics
+        default_dimensions: passed to aws_lambda_powertools.Metrics
         model: passed to aws_lambda_powertools.utilities.parser.event_parser
         envelope: passed to aws_lambda_powertools.utilities.parser.event_parser
 
@@ -584,7 +657,7 @@ def powertools(
                 lambda_handler=func,
                 capture_cold_start_metric=capture_cold_start_metric,
                 raise_on_empty_metrics=raise_on_empty_metrics,
-                default_dimensions=default_dimentions,
+                default_dimensions=default_dimensions,
             )
 
         if model is not None:
